@@ -21,6 +21,7 @@ import os.path
 import platform
 from enum import Enum
 from io import BytesIO
+import logging
 
 have_img2pdf = True
 try:
@@ -32,6 +33,7 @@ have_tkinter = True
 try:
     import tkinter
     import tkinter.filedialog
+    import tkinter.messagebox
 except ImportError:
     have_tkinter = False
 
@@ -44,7 +46,7 @@ except ImportError:
     tkinter.Menubutton = dummy
     tkinter.LabelFrame = dummy
 
-VERSION = "0.3"
+VERSION = "0.4"
 
 PAGE_SIZES = OrderedDict(
     [
@@ -97,7 +99,7 @@ def cm_to_mm(length):
 
 
 def in_to_mm(length):
-    return length / 25.4
+    return length * 25.4
 
 
 def pt_to_mm(length):
@@ -300,20 +302,8 @@ def complex_cover(n, m, x, y):
 
 
 class Plakativ:
-    def __init__(self, infile, pagenr=0):
-        self.doc = None
-        if have_img2pdf:
-            # if we have img2pdf available we can encapsulate a raster image
-            # into a PDF container
-            try:
-                data = img2pdf.convert(infile)
-            except img2pdf.ImageOpenError:
-                # img2pdf cannot handle this
-                pass
-            else:
-                stream = BytesIO()
-                stream.write(data)
-                self.doc = fitz.open(stream=stream, filetype="application/pdf")
+    def __init__(self, doc=None, pagenr=0):
+        self.doc = doc
         if self.doc is None:
             # either we didn't have img2pdf or opening the input with img2pdf
             # failed
@@ -631,7 +621,7 @@ class Plakativ:
 
         return postersize, mult, npages
 
-    def render(self, outfile):
+    def render(self, outfile, cover=False, guides=False, numbers=False, border=False):
         if not hasattr(self, "layout"):
             raise LayoutNotComputedException()
 
@@ -639,7 +629,85 @@ class Plakativ:
 
         outdoc = fitz.open()
 
-        for (x, y, portrait) in self.layout["positions"]:
+        if cover:
+            # factor to convert from output poster dimensions (given in mm) into
+            # pdf dimensions (given in pt)
+            zoom_1 = min(
+                mm_to_pt(
+                    self.layout["output_pagesize"][0]
+                    - 2 * max(self.layout["border_left"], self.layout["border_right"])
+                )
+                / (self.layout["overallsize"][0]),
+                mm_to_pt(
+                    self.layout["output_pagesize"][1]
+                    - 2 * max(self.layout["border_top"], self.layout["border_bottom"])
+                )
+                / (self.layout["overallsize"][1]),
+            )
+            page = outdoc.newPage(
+                -1,  # insert after last page
+                width=mm_to_pt(self.layout["output_pagesize"][0]),
+                height=mm_to_pt(self.layout["output_pagesize"][1]),
+            )
+            for i, (x, y, portrait) in enumerate(self.layout["positions"]):
+                x0 = (x + self.layout["posterpos"][0]) * zoom_1 + (
+                    mm_to_pt(self.layout["output_pagesize"][0])
+                    - zoom_1 * self.layout["overallsize"][0]
+                ) / 2
+                y0 = (y + self.layout["posterpos"][1]) * zoom_1 + (
+                    mm_to_pt(self.layout["output_pagesize"][1])
+                    - zoom_1 * self.layout["overallsize"][1]
+                ) / 2
+                if portrait:
+                    page_width = self.layout["output_pagesize"][0] * zoom_1
+                    page_height = self.layout["output_pagesize"][1] * zoom_1
+                    top = self.layout["border_top"] * zoom_1
+                    right = self.layout["border_right"] * zoom_1
+                    bottom = self.layout["border_bottom"] * zoom_1
+                    left = self.layout["border_left"] * zoom_1
+                else:
+                    # page is rotated 90 degrees clockwise
+                    page_width = self.layout["output_pagesize"][1] * zoom_1
+                    page_height = self.layout["output_pagesize"][0] * zoom_1
+                    top = self.layout["border_left"] * zoom_1
+                    right = self.layout["border_top"] * zoom_1
+                    bottom = self.layout["border_right"] * zoom_1
+                    left = self.layout["border_bottom"] * zoom_1
+                # inner rectangle
+                shape = page.newShape()
+                shape.drawRect(
+                    fitz.Rect(
+                        x0,
+                        y0,
+                        x0 + page_width - left - right,
+                        y0 + page_height - top - bottom,
+                    )
+                )
+                shape.finish(color=(0, 0, 1))
+                # outer rectangle
+                shape.drawRect(
+                    fitz.Rect(
+                        x0 - left,
+                        y0 - top,
+                        x0 - left + page_width,
+                        y0 - top + page_height,
+                    )
+                )
+                shape.finish(color=(1, 0, 0))
+                shape.insertTextbox(
+                    fitz.Rect(
+                        x0 + 5,
+                        y0 + 5,
+                        x0 + page_width - left - right - 5,
+                        y0 + page_height - top - bottom - 5,
+                    ),
+                    "%d" % (i + 1),
+                    fontsize=20,
+                    color=(0, 0, 0),
+                )
+                shape.commit()
+
+        for i, (x, y, portrait) in enumerate(self.layout["positions"]):
             if portrait:
                 page_width = mm_to_pt(self.layout["output_pagesize"][0])
                 page_height = mm_to_pt(self.layout["output_pagesize"][1])
@@ -697,6 +765,90 @@ class Plakativ:
                 clip=sourcerect,  # part of the input page to use
             )
 
+            shape = page.newShape()
+            if guides:
+                if portrait:
+                    shape.drawRect(
+                        fitz.Rect(
+                            mm_to_pt(self.layout["border_left"]),
+                            mm_to_pt(self.layout["border_top"]),
+                            page_width - mm_to_pt(self.layout["border_right"]),
+                            page_height - mm_to_pt(self.layout["border_bottom"]),
+                        )
+                    )
+                else:
+                    shape.drawRect(
+                        fitz.Rect(
+                            mm_to_pt(self.layout["border_bottom"]),
+                            mm_to_pt(self.layout["border_left"]),
+                            page_width - mm_to_pt(self.layout["border_top"]),
+                            page_height - mm_to_pt(self.layout["border_right"]),
+                        )
+                    )
+                shape.finish(width=0.2, color=(0.5, 0.5, 0.5), dashes="[5 6 1 6] 0")
+            if numbers:
+                if portrait:
+                    shape.insertTextbox(
+                        fitz.Rect(
+                            mm_to_pt(self.layout["border_left"]) + 5,
+                            mm_to_pt(self.layout["border_top"]) + 5,
+                            page_width - mm_to_pt(self.layout["border_right"]) - 5,
+                            page_height - mm_to_pt(self.layout["border_bottom"]) - 5,
+                        ),
+                        "%d" % (i + 1),
+                        fontsize=8,
+                        color=(0.5, 0.5, 0.5),
+                    )
+                else:
+                    shape.insertTextbox(
+                        fitz.Rect(
+                            mm_to_pt(self.layout["border_bottom"]) + 5,
+                            mm_to_pt(self.layout["border_left"]) + 5,
+                            page_width - mm_to_pt(self.layout["border_top"]) - 5,
+                            page_height - mm_to_pt(self.layout["border_right"]) - 5,
+                        ),
+                        "%d" % (i + 1),
+                        fontsize=8,
+                        color=(0.5, 0.5, 0.5),
+                    )
+            if border:
+                if portrait:
+                    shape.drawRect(
+                        fitz.Rect(
+                            mm_to_pt(self.layout["border_left"] - x),
+                            mm_to_pt(self.layout["border_top"] - y),
+                            mm_to_pt(
+                                self.layout["border_left"]
+                                - x
+                                + self.layout["postersize"][0]
+                            ),
+                            mm_to_pt(
+                                self.layout["border_top"]
+                                - y
+                                + self.layout["postersize"][1]
+                            ),
+                        )
+                    )
+                else:
+                    shape.drawRect(
+                        fitz.Rect(
+                            mm_to_pt(self.layout["border_bottom"] - x),
+                            mm_to_pt(self.layout["border_left"] - y),
+                            mm_to_pt(
+                                self.layout["border_bottom"]
+                                - x
+                                + self.layout["postersize"][0]
+                            ),
+                            mm_to_pt(
+                                self.layout["border_left"]
+                                - y
+                                + self.layout["postersize"][1]
+                            ),
+                        )
+                    )
+                shape.finish(width=0.2, color=(0.5, 0.5, 0.5), dashes="[1 1] 0")
+            shape.commit()
+
         if hasattr(outfile, "write"):
             # outfile is an object with a write() method
             outfile.write(outdoc.write(garbage=4, deflate=True))
@@ -818,16 +970,13 @@ class OptionMenu(tkinter.Menubutton):
 
 
 class Application(tkinter.Frame):
-    def __init__(self, master=None, plakativ=None):
+    def __init__(self, master=None):
         super().__init__(master)
         self.master = master
         self.master.title("plakativ")
 
         self.pack(fill=tkinter.BOTH, expand=tkinter.TRUE)
 
-        if plakativ is not None:
-            self.plakativ = plakativ
-
         self.canvas = tkinter.Canvas(self, bg="black")
         self.canvas.pack(fill=tkinter.BOTH, side=tkinter.LEFT, expand=tkinter.TRUE)
         self.canvas_size = self.canvas.winfo_width(), self.canvas.winfo_height()
@@ -859,13 +1008,13 @@ class Application(tkinter.Frame):
         self.pagesize.pack(fill=tkinter.X)
         self.pagesize.set(False, (210, 297))
         if hasattr(self, "plakativ"):
-            self.postersize.callback = self.on_pagesize
+            self.pagesize.callback = self.on_pagesize
 
         self.bordersize = BorderSizeWidget(frame1.interior)
         self.bordersize.pack(fill=tkinter.X)
         self.bordersize.set(15.0, 15.0, 15.0, 15.0)
         if hasattr(self, "plakativ"):
-            self.postersize.callback = self.on_bordersize
+            self.bordersize.callback = self.on_bordersize
 
         self.postersize = PostersizeWidget(frame1.interior)
         self.postersize.pack(fill=tkinter.X)
@@ -877,23 +1026,10 @@ class Application(tkinter.Frame):
         self.layouter.pack(fill=tkinter.X)
         self.layouter.set("simple")
         if hasattr(self, "plakativ"):
-            self.postersize.callback = self.on_layouter
+            self.layouter.callback = self.on_layouter
 
-        output_group = tkinter.LabelFrame(frame1.interior, text="Output options")
-        output_group.pack(fill=tkinter.X)
-
-        tkinter.Checkbutton(
-            output_group, text="Print cutting guides", state=tkinter.DISABLED
-        ).pack(anchor=tkinter.W)
-        tkinter.Checkbutton(
-            output_group, text="Print poster border", state=tkinter.DISABLED
-        ).pack(anchor=tkinter.W)
-        tkinter.Checkbutton(
-            output_group, text="Print page number", state=tkinter.DISABLED
-        ).pack(anchor=tkinter.W)
-        tkinter.Checkbutton(
-            output_group, text="Print layout cover page", state=tkinter.DISABLED
-        ).pack(anchor=tkinter.W)
+        self.outopts = OutOptsWidget(frame1.interior)
+        self.outopts.pack(fill=tkinter.X)
 
         option_group = tkinter.LabelFrame(frame1.interior, text="Program options")
         option_group.pack(fill=tkinter.X)
@@ -1109,22 +1245,74 @@ class Application(tkinter.Frame):
         # print("saved ", filename)
 
     def on_open_button(self):
+        if have_img2pdf:
+            filetypes = [
+                ("all supported", "*.pdf *.png *.jpg *.jpeg *.gif *.tiff *.tif"),
+                ("pdf documents", "*.pdf"),
+                ("png images", "*.png"),
+                ("jpg images", "*.jpg *.jpeg"),
+                ("gif images", "*.gif"),
+                ("tiff images", "*.tiff *.tif"),
+                ("all files", "*"),
+            ]
+        else:
+            filetypes = [
+                ("pdf documents", "*.pdf"),
+                ("all files", "*"),
+            ]
         filename = tkinter.filedialog.askopenfilename(
             parent=self.master,
             title="Open either a PDF or a raster image",
-            filetypes=[
-                ("pdf documents", "*.pdf"),
-                ("png images", "*.png"),
-                ("jpg images", "*.jpg"),
-                ("all files", "*"),
-            ],
+            filetypes=filetypes
             # initialdir="/home/josch/git/plakativ",
             # initialfile="test.pdf",
         )
         if filename == ():
             return
+        self.open_file(filename)
+
+    def open_file(self, filename):
         self.filename = filename
-        self.plakativ = Plakativ(self.filename)
+        doc = None
+        if have_img2pdf:
+            # if we have img2pdf available we can encapsulate a raster image
+            # into a PDF container
+            data = None
+            try:
+                data = img2pdf.convert(self.filename)
+            except img2pdf.AlphaChannelError:
+                remove_alpha = tkinter.messagebox.askyesno(
+                    title="Removing Alpha Channel",
+                    message="PDF does not support alpha channels. Should the "
+                    "alpha channel be removed? The resulting PDF might not be "
+                    "lossless anymore.",
+                )
+                # remove alpha channel
+                if remove_alpha:
+                    from PIL import Image
+
+                    img = Image.open(self.filename).convert("RGBA")
+                    background = Image.new("RGBA", img.size, (255, 255, 255))
+                    img = Image.alpha_composite(background, img)
+                    with BytesIO() as output:
+                        img.convert("RGB").save(output, format="PNG")
+                        output.seek(0)
+                        data = img2pdf.convert(output)
+                else:
+                    return
+            except img2pdf.ImageOpenError:
+                # img2pdf cannot handle this
+                pass
+
+            if data is not None:
+                stream = BytesIO()
+                stream.write(data)
+                doc = fitz.open(stream=stream, filetype="application/pdf")
+        if doc is None:
+            # either we didn't have img2pdf or opening the input with img2pdf
+            # failed
+            doc = fitz.open(filename=self.filename)
+        self.plakativ = Plakativ(doc)
         # compute the splitting with the current values
         mode, (custom_size, size), mult, npages = self.postersize.value
         _, pagesize = self.pagesize.value
@@ -1167,7 +1355,13 @@ class Application(tkinter.Frame):
         )
         if filename == "":
             return
-        self.plakativ.render(filename)
+        self.plakativ.render(
+            filename,
+            cover=self.outopts.variables["cover"].get(),
+            guides=self.outopts.variables["guides"].get(),
+            numbers=self.outopts.variables["numbers"].get(),
+            border=self.outopts.variables["border"].get(),
+        )
 
 
 class LayouterWidget(tkinter.LabelFrame):
@@ -1214,6 +1408,31 @@ class LayouterWidget(tkinter.LabelFrame):
             self.variables["strategy"].set(strategy)
 
 
+class OutOptsWidget(tkinter.LabelFrame):
+    def __init__(self, parent, *args, **kw):
+        tkinter.LabelFrame.__init__(self, parent, text="Output options", *args, **kw)
+
+        self.variables = {
+            "guides": tkinter.IntVar(),
+            "border": tkinter.IntVar(),
+            "numbers": tkinter.IntVar(),
+            "cover": tkinter.IntVar(),
+        }
+
+        tkinter.Checkbutton(
+            self, text="Print cutting guides", variable=self.variables["guides"]
+        ).pack(anchor=tkinter.W)
+        tkinter.Checkbutton(
+            self, text="Print poster border", variable=self.variables["border"]
+        ).pack(anchor=tkinter.W)
+        tkinter.Checkbutton(
+            self, text="Print page number", variable=self.variables["numbers"]
+        ).pack(anchor=tkinter.W)
+        tkinter.Checkbutton(
+            self, text="Print layout cover page", variable=self.variables["cover"]
+        ).pack(anchor=tkinter.W)
+
+
 class InputWidget(tkinter.LabelFrame):
     def __init__(self, parent, *args, **kw):
         tkinter.LabelFrame.__init__(self, parent, text="Input properties", *args, **kw)
@@ -1581,7 +1800,7 @@ class PostersizeWidget(tkinter.LabelFrame):
         ).grid(row=4, column=0, columnspan=3, sticky=tkinter.W)
         tkinter.Label(
             self,
-            text="Multipler:",
+            text="Multiplier:",
             state=tkinter.DISABLED,
             name="mult_label_multiplier",
         ).grid(row=5, column=0, sticky=tkinter.W, padx=(27, 0))
@@ -1596,9 +1815,6 @@ class PostersizeWidget(tkinter.LabelFrame):
             state=tkinter.DISABLED,
             name="mult_spinbox_multiplier",
         ).grid(row=5, column=1, sticky=tkinter.W)
-        tkinter.Label(
-            self, text="%", state=tkinter.DISABLED, name="mult_label_perc"
-        ).grid(row=5, column=2, sticky=tkinter.W)
 
         tkinter.Radiobutton(
             self,
@@ -1725,17 +1941,77 @@ def compute_layout(
     pagesize=(210, 297),
     border=(0, 0, 0, 0),
     strategy="simple",
+    remove_alpha=False,
+    cover=False,
+    guides=False,
+    numbers=False,
+    poster_border=False,
 ):
-    plakativ = Plakativ(infile, pagenr)
+    doc = None
+    if hasattr(infile, "read"):
+        # we have to slurp in the whole file because we potentially read it
+        # in multiple times in case img2pdf is installed
+        # also, mupdf needs to be able to seek(), so we need to slurp it in
+        # anyways
+        infile = BytesIO(infile.read())
+    if have_img2pdf:
+        # if we have img2pdf available we can encapsulate a raster image
+        # into a PDF container
+        data = None
+        try:
+            # FIXME: img2pdf should not use the root logger so that instead we
+            #        can run logging.getLogger('img2pdf').setLevel(logging.CRITICAL)
+            logging.getLogger().setLevel(logging.CRITICAL)
+            data = img2pdf.convert(infile)
+        except img2pdf.AlphaChannelError:
+            if remove_alpha:
+                # remove alpha channel
+                from PIL import Image
+
+                img = Image.open(infile).convert("RGBA")
+                background = Image.new("RGBA", img.size, (255, 255, 255))
+                img = Image.alpha_composite(background, img)
+                with BytesIO() as output:
+                    img.convert("RGB").save(output, format="PNG")
+                    output.seek(0)
+                    data = img2pdf.convert(output)
+            else:
+                print(
+                    """
+Plakativ is lossless by default. To automatically remove the alpha channel from
+the input and place the image on a white background, use the --remove-alpha
+option""",
+                    file=sys.stderr,
+                )
+                exit(1)
+        except img2pdf.ImageOpenError:
+            # img2pdf cannot handle this
+            pass
+
+        if data is not None:
+            stream = BytesIO()
+            stream.write(data)
+            doc = fitz.open(stream=stream, filetype="application/pdf")
+    if doc is None:
+        # either we didn't have img2pdf or opening the input with img2pdf
+        # failed
+        if hasattr(infile, "read"):
+            doc = fitz.open(stream=infile, filetype="application/pdf")
+        else:
+            doc = fitz.open(filename=infile)
+    plakativ = Plakativ(doc, pagenr)
     plakativ.compute_layout(mode, size, mult, npages, pagesize, border, strategy)
-    plakativ.render(outfile)
+    plakativ.render(outfile, cover, guides, numbers, poster_border)
+    doc.close()
 
 
-def gui():
+def gui(filename=None):
     if not have_tkinter:
         raise Exception("the GUI requires tkinter")
     root = tkinter.Tk()
     app = Application(master=root)
+    if filename is not None:
+        app.open_file(filename)
     app.mainloop()
 
 
@@ -1761,7 +2037,7 @@ def parse_num(num, name):
             )
             raise argparse.ArgumentTypeError(msg)
     if unit is None:
-        unit = Unit.pt
+        unit = Unit.mm
     else:
         num = num[:-2]
         try:
@@ -1828,6 +2104,18 @@ def parse_pagesize_rectarg(string):
 
 
 def main():
+    if len(sys.argv) == 1 and platform.system() != "Windows":
+        print(
+            """
+You called plakativ without arguments. At least one of the options --size,
+--factor or --maxpages is required for running plakativ on the command line.
+But maybe you meant to run the plakativ GUI instead? On platforms other than
+Windows, the default is to run the command line interface. To run the graphical
+user interface, run plakativ with the --gui option instead.
+""",
+            file=sys.stderr,
+        )
+
     rendered_papersizes = ""
     for k, v in sorted(papersizes.items()):
         rendered_papersizes += "    %-8s %s\n" % (papernames[k], v)
@@ -1848,6 +2136,7 @@ glued together to form a larger poster. Features:
   - GUI with preview functionality
   - complex layouter to save paper
   - optimize by number of pages, output poster size or multiple of input area
+  - support for raster images as input if img2pdf is available
 
 Options:
 """,
@@ -1906,6 +2195,13 @@ This will create a file poster.pdf with multiple DIN A4 pages which, after
 being cut and glued together will form a DIN A1 poster of the content on the
 first page of input.pdf.
 
+If img2pdf is available as a Python module, then plakativ can also use raster
+images as input. Since img2pdf refuses to work on images with an alpha channel,
+you can instruct plakativ to remove the alpha channel for you with the
+--remove-alpha flag:
+
+    $ plakativ --size A1 --output=poster.pdf --remove-alpha input.png
+
 Written by Johannes 'josch' Schauer <josch@mister-muffin.de>
 
 Report bugs at https://gitlab.mister-muffin.de/josch/plakativ/issues
@@ -1996,7 +2292,8 @@ Report bugs at https://gitlab.mister-muffin.de/josch/plakativ/issues
         help="The borders on each output page for gluing. This specifies how "
         "much the pages overlap each other. If your printer cannot print "
         "borderless, then this value should also be larger than the border up "
-        "to which your printer is able to print. One value sets the border on "
+        "to which your printer is able to print. The default unit is mm. "
+        "Other possible units are cm, in and pt. One value sets the border on "
         "all four sides. Multiple values are separated by a colon. With two "
         "values, the first value sets top and bottom border and the second "
         "value sets left and right border. With three values, the first value "
@@ -2013,11 +2310,48 @@ Report bugs at https://gitlab.mister-muffin.de/josch/plakativ/issues
         "layout is able to sometimes require less pages for the same poster "
         "size and is allowed to rotate pages.",
     )
+    parser.add_argument(
+        "--remove-alpha",
+        action="store_true",
+        help="When the input is a raster image instead of a PDF document, "
+        "plakativ can remove the alpha channel for you. The resulting PDF "
+        "poster might not be lossless anymore.",
+    )
+    parser.add_argument(
+        "--cover-page",
+        action="store_true",
+        help="Add a cover page as the first page which shows the resulting "
+        "layout. This is especially interesting for the complex layouter "
+        "unless you like puzzles.",
+    )
+    parser.add_argument(
+        "--cutting-guides",
+        action="store_true",
+        help="Print light-gray dashed lines that surround the visible part "
+        "of each page and can help with easier cutting and gluing of the "
+        "pages. This is generally only needed if the poster does not contain "
+        "enough detail for accurate gluing.",
+    )
+    parser.add_argument(
+        "--page-numbers",
+        action="store_true",
+        help="Print a small number of each page to uniquely identify each "
+        "sheet. This is especially useful in combination with --cover-page "
+        "because the numbers on the cover page correspond to the page numbers.",
+    )
+    parser.add_argument(
+        "--poster-border",
+        action="store_true",
+        help="If the poster itself has a white background and it is important "
+        "that the final result has precisely the desired poster size, then "
+        "this option will print a light-gray dashed border around the whole "
+        "poster, so that it can be accurately cut to the correct overall size.",
+    )
 
     args = parser.parse_args()
 
     if args.gui:
-        gui()
+        gui(args.input)
         sys.exit(0)
 
     if not args.input or args.input == "-":
@@ -2033,7 +2367,12 @@ Report bugs at https://gitlab.mister-muffin.de/josch/plakativ/issues
     elif isinstance(args.mode, int):
         mode = "npages"
     else:
-        raise Exception("logic error")
+        print(
+            "Error: must supply one of --size, --factor or --maxpages\n",
+            file=sys.stderr,
+        )
+        parser.print_usage(sys.stderr)
+        sys.exit(1)
 
     compute_layout(
         args.input,
@@ -2044,6 +2383,11 @@ Report bugs at https://gitlab.mister-muffin.de/josch/plakativ/issues
         border=args.border,
         strategy=args.layouter,
         **{mode: args.mode},
+        remove_alpha=args.remove_alpha,
+        cover=args.cover_page,
+        guides=args.cutting_guides,
+        numbers=args.page_numbers,
+        poster_border=args.poster_border,
     )
 
 
